@@ -1,5 +1,10 @@
 /*
- * Copyright (C) 2003 _INRIA_
+ * Copyright (C) 2003 INRIA
+ *
+ *	INRIA
+ *	Domaine de Voluceau
+ *	Rocquencourt - B.P. 105
+ *	78153 Le Chesnay Cedex - France
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,54 +28,34 @@
 #define _RFID_H_
 
 #include <signal.h>
-#include <s6350.h>
-
-/*
- * Assume that a RFID transponder can hold a maximum of 32 bytes.
- */
-#define RFID_TRANSPONDER_DATA_SIZE 32
-/*
- * Size of a string holding hexadecimal representation of 
- * a RFID transponder identity, with separators and trailing 
- * NULL.
- */
-#define RFID_TRANSPONDER_HEX_ID_SIZE 32
-
-/*
- * RFID transponder identity and data.
- */
-typedef struct {
-  s6350_transponder_t info;		/* Identity and storage
-					   layout. */
-  u_int8_t data[RFID_TRANSPONDER_DATA_SIZE + 1]; /* + 1 for trailing NULL */
-  int data_length;			/* Length of data in the
-					   data field. */
-} rfid_transponder_t;
+#include <rfid_reader.h>
 
 /*
  * Values of the options field of the rfid_t structure.
  */
 /*
- * If set, the RFID transponder will polled every 5 milliseconds and
- * will be reported each time it is found.  If not set, a RFID
- * transponder placed on a RFID reader is only reported once.
- * For instance, the rfid_read function will only call the callback
- * once.
+ * If set, the RFID reader is assumed to respond to the ISO-15695-3
+ * protocol.
  */
-#define RFID_OPTION_POLL		0x00000001
+#define RFID_OPTION_ISO15693		0x00000002
+/*
+ * If set, the frames sent/received to/from the RFID reader
+ * are dumped on stderr.
+ */
+#define RFID_OPTION_DUMP_FRAMES		0x00000004
 
 /*
  * Handler holding context for a specific RFID device and used by
  * all rfid functions.
  */
 typedef struct {
-  s6350_t s6350;		/* S6350 Midrange Reader handler. */
+  rfid_reader_t* reader;	/* RFID reader handler. */
   int options;			/* Flags controling the behavior of
 				   rfid functions. */
   int error;			/* Errors set by rfid functions. */
   char* device;			/* Path name of the device connecting
 				   to the RFID reader. */
-  rfid_transponder_t last;	/* Copy of the latest RFID transponder
+  rfid_transponder_t* last;	/* Copy of the latest RFID transponder
 				   reported by the RFID reader. */
   int verbose;			/* Control verbosity level of
 				   functions. */
@@ -96,6 +81,13 @@ void rfid_end(rfid_t* rfid);
 #define RFID_CALLBACK_END	1	/* Everything is fine, end outer loop. */
 #define RFID_CALLBACK_ERROR	-1	/* Something went wrong, end outer loop on error. */
 
+#define RFID_EVENT_NEW		0x01	
+#define RFID_EVENT_DELETE	0x02	
+#define RFID_EVENT_AGAIN	0x04
+#define RFID_EVENT_INVENTORY	0x08
+#define RFID_EVENT_ALL		0xFF
+#define RFID_EVENT_NONE		0x00
+
 /*
  * Callback function for the rfid_read function.
  * Each time a transponder is found, call the callback with transponder information
@@ -105,7 +97,17 @@ void rfid_end(rfid_t* rfid);
  * The transponder->data array is always NULL terminated.
  * 
  */
-typedef int rfid_read_callback_t(rfid_t* rfid, const rfid_transponder_t* transponder, void* data);
+typedef int rfid_transponder_callback_t(rfid_t* rfid, int event, const rfid_transponder_t* transponder, void* data);
+
+typedef int rfid_inventory_callback_t(rfid_t* rfid, rfid_transponder_t** transponders, int transponders_length, void* data);
+
+/*
+ */
+rfid_transponder_t* rfid_transponder_alloc(rfid_t* rfid);
+void rfid_transponder_free(rfid_t* rfid, rfid_transponder_t* transponder);
+char* rfid_transponder_describe(rfid_t* rfid, rfid_transponder_t* transponder);
+int rfid_transponder_id_get(rfid_t* rfid, rfid_transponder_t* transponder, char** idp);
+int rfid_transponder_id_set(rfid_t* rfid, rfid_transponder_t* transponder, char* id);
 
 /*
  * Infinite loop polling the RFID reader (using rfid_poll). When
@@ -116,17 +118,7 @@ typedef int rfid_read_callback_t(rfid_t* rfid, const rfid_transponder_t* transpo
  * function is only called if a RFID transponder change occurs. 
  *
  */
-int rfid_read(rfid_t* rfid, rfid_read_callback_t* callback, void* data);
-
-/*
- * Query the RFID reader for a transponder. If one is found, fill the
- * transponder argument with information and content. Return on error
- * if no transponder is found, and set rfid->error to ENOENT.
- *
- * The transponder->data array is always NULL terminated.
- * 
- */
-int rfid_poll(rfid_t* rfid, rfid_transponder_t* transponder);
+int rfid_event_loop(rfid_t* rfid, int events_mask, rfid_transponder_callback_t* transponder_callback, rfid_inventory_callback_t* inventory_callback, void* data);
 
 /*
  * Write transponder->data_length bytes from transponder->data to
@@ -151,11 +143,6 @@ char* rfid_strerror(rfid_t* rfid);
  */
 int rfid_error(rfid_t* rfid);
 
-/*
- * Return the file descriptor used to dialog with the RFID reader.
- */
-#define rfid_fd(rfid) ((rfid)->s6350.fd)
-
 /*
  * Set the options to value.
  */
