@@ -1,5 +1,10 @@
 /*
- * Copyright (C) 2003 _INRIA_
+ * Copyright (C) 2003 INRIA
+ *
+ *	INRIA
+ *	Domaine de Voluceau
+ *	Rocquencourt - B.P. 105
+ *	78153 Le Chesnay Cedex - France
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +45,7 @@ typedef struct {
   rfid_t* rfid;
   int verbose;
   int max_count;
+  int iso;
 } commander_t;
 
 /*
@@ -56,7 +62,7 @@ static void report_error(rfid_t* rfid, const char* message)
  * Implements --run action when a RFID transponder is found.
  * Call system(3) on RFID transponder data.
  */
-static int callback(rfid_t* rfid, const rfid_transponder_t* transponder, void* data)
+static int transponder_callback(rfid_t* rfid, int event, const rfid_transponder_t* transponder, void* data)
 {
   commander_t* commander = (commander_t*)data;
 
@@ -64,25 +70,12 @@ static int callback(rfid_t* rfid, const rfid_transponder_t* transponder, void* d
     commander->max_count--;
 
   if(commander->verbose > 1) {
-    const s6350_transponder_t* info = &transponder->info;
+    char* transponder_description = rfid->reader->transponder_describe_f(rfid->reader, transponder);
+
     fprintf(stderr, "\
 ========================================\n\
-transponder_id = 0x%02x%02x%02x%02x\n\
-manufacturer = 0x%02x\n\
-version_number = 0x%04x\n\
-blocks = %d\n\
-bytes_per_block = %d\n\
-",
-	    info->transponder_id[3],
-	    info->transponder_id[2],
-	    info->transponder_id[1],
-	    info->transponder_id[0],
-	    info->manufacturer,
-	    info->version_number,
-	    info->blocks,
-	    info->bytes_per_block
-	    );
-
+%s",
+	  transponder_description);
     {
       int i;
       for(i = 0; i < transponder->data_length; i++)
@@ -108,7 +101,7 @@ bytes_per_block = %d\n\
  */
 static int run_daemon(commander_t* commander)
 {
-  if(rfid_read(commander->rfid, callback, commander) < 0) {
+  if(rfid_event_loop(commander->rfid, RFID_EVENT_NEW, transponder_callback, 0, commander) < 0) {
     report_error(commander->rfid, "rfid_read");
     return -1;
   }
@@ -165,13 +158,14 @@ storage (maximum %d bytes).\n", transponder.data_length, RFID_TRANSPONDER_DATA_S
 static void usage(const char* message)
 {
   fprintf(stderr, "%s\n\n\
-rfid_commander [--help] [--device special] [--verbose] [--count n] [--run]\n\
-rfid_commander [--help] [--device special] [--verbose] [--add] cmd arg arg ...\n\
+rfid_commander [--help] [--device special] [--verbose] [--iso] [--count n] [--run]\n\
+rfid_commander [--help] [--device special] [--verbose] [--iso] [--add] cmd arg arg ...\n\
     --run              wait for a transponder and run the command\n\
                        memorized in it. Loop forever.\n\
     --count n          loop n times instead of looping forever.\n\
     --add              wait for a transponder and store the command\n\
                        given in argument in its memory.\n\
+    --iso              interact with ISO-15693 conformant transponder.\n\
     --device special   dialog with RFID reader connected to special\n\
                        file <special> (default /dev/ttyS0)\n\
     --help             print this message\n\
@@ -195,6 +189,7 @@ int main(int argc, char** argv) {
 
   commander.rfid = &rfid;
   commander.verbose = 0;
+  commander.iso = 0;
   commander.max_count = -1;
 
   while (1) {
@@ -202,6 +197,7 @@ int main(int argc, char** argv) {
     static struct option long_options[] = {
       {"add", 0, 0, 'a'},
       {"run", 0, 0, 'r'},
+      {"iso", 0, 0, 'i'},
       {"device", 1, 0, 'd'},
       {"count", 1, 0, 'c'},
       {"help", 0, 0, 'h'},
@@ -209,7 +205,7 @@ int main(int argc, char** argv) {
       {0, 0, 0, 0}
     };
 
-    c = getopt_long (argc, argv, "arvd:c:", long_options, &option_index);
+    c = getopt_long (argc, argv, "arivd:c:", long_options, &option_index);
     if (c == -1)
       break;
 
@@ -222,6 +218,10 @@ int main(int argc, char** argv) {
       option_add++;
       break;
 
+    case 'i':
+      commander.iso++;
+      break;
+
     case 'c':
       commander.max_count = atoi(optarg);
       break;
@@ -257,8 +257,14 @@ int main(int argc, char** argv) {
     return -1;
   }
   
+  if(commander.iso)
+    rfid.options |= RFID_OPTION_ISO15693;
+
   rfid.verbose = commander.verbose;
 
+  if(commander.verbose > 2)
+    rfid.options |= RFID_OPTION_DUMP_FRAMES;
+
   if(rfid_init(&rfid) < 0) {
     report_error(&rfid, "rfid_init");
     return -1;
