@@ -1,5 +1,10 @@
 /*
- * Copyright (C) 2003 _INRIA_
+ * Copyright (C) 2003 INRIA
+ *
+ *	INRIA
+ *	Domaine de Voluceau
+ *	Rocquencourt - B.P. 105
+ *	78153 Le Chesnay Cedex - France
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -77,31 +82,19 @@ FAILURE: Make sure a transponder is placed on the RFID reader.\n\
 
 static char* test_string = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
-static int callback(rfid_t* rfid, const rfid_transponder_t* transponder, void* data)
+static int transponder_callback(rfid_t* rfid, int event, const rfid_transponder_t* transponder, void* data)
 {
   int* count = (int*)data;
   (*count)--;
-  
-  const s6350_transponder_t* info = &transponder->info;
+
+  char* transponder_description = rfid->reader->transponder_describe_f(rfid->reader, transponder);
   fprintf(stderr, "\
 ========================================\n\
-transponder_id = 0x%02x%02x%02x%02x\n\
-manufacturer = 0x%02x\n\
-version_number = 0x%04x\n\
-blocks = %d\n\
-bytes_per_block = %d\n\
-",
-	  info->transponder_id[3],
-	  info->transponder_id[2],
-	  info->transponder_id[1],
-	  info->transponder_id[0],
-	  info->manufacturer,
-	  info->version_number,
-	  info->blocks,
-	  info->bytes_per_block
-	  );
-
+%s",
+	  transponder_description);
 
+  free(transponder_description);
+  
   {
     int i;
     for(i = 0; i < transponder->data_length; i++)
@@ -122,20 +115,22 @@ bytes_per_block = %d\n\
 
 int main(int argc, char** argv)
 {
+  int timeout = 0;
 
   if(argc > 1)
     rfid.device = argv[1];
   else
     rfid.device = "/dev/ttyS0";
 
-  rfid_options(&rfid, RFID_OPTION_POLL);
+  if(argc > 2)
+    rfid_options(&rfid, RFID_OPTION_ISO15693);
 
   if(signal(SIGALRM, timeout_init) < 0) {
     perror("signal(SIGALRM)");
     return -1;
   }
 
-  alarm(5);
+  alarm(timeout);
   if(rfid_init(&rfid) < 0) {
     report_error(&rfid, "rfid_init");
     return -1;
@@ -147,50 +142,44 @@ int main(int argc, char** argv)
     return -1;
   }
 
-  alarm(5);
+  alarm(timeout);
   {
-    rfid_transponder_t transponder;
-    const s6350_transponder_t* info = &transponder.info;
-    memset(&transponder, '\0', sizeof(rfid_transponder_t));
-    memcpy(transponder.data, test_string, strlen(test_string));
-    transponder.data_length = strlen(test_string);
-    if(rfid_write(&rfid, &transponder) < 0) {
+    rfid_transponder_t* transponder = rfid_transponder_alloc(&rfid);
+    char* transponder_description = 0;
+    
+    memcpy(transponder->data, test_string, strlen(test_string));
+    transponder->data_length = strlen(test_string);
+    if(rfid_write(&rfid, transponder) < 0) {
       report_error(&rfid, "rfid_write");
+      rfid_transponder_free(&rfid, transponder);
       return -1;
     }
 
+    transponder_description = rfid_transponder_describe(&rfid, transponder);
+
     fprintf(stderr, "\
 ========================================\n\
 Wrote string %s to transponder:\n\
-transponder_id = 0x%02x%02x%02x%02x\n\
-manufacturer = 0x%02x\n\
-version_number = 0x%04x\n\
-blocks = %d\n\
-bytes_per_block = %d\n\
-",
+%s",
 	    test_string,
-	    info->transponder_id[3],
-	    info->transponder_id[2],
-	    info->transponder_id[1],
-	    info->transponder_id[0],
-	    info->manufacturer,
-	    info->version_number,
-	    info->blocks,
-	    info->bytes_per_block
-	    );
+	    transponder_description);
+    free(transponder_description);
+    rfid_transponder_free(&rfid, transponder);
   }
-  alarm(5);
+  alarm(0);
+
+  alarm(timeout);
   {
-    rfid_transponder_t transponder;
-    s6350_transponder_t* info = &transponder.info;
-    memset(&transponder, '\0', sizeof(rfid_transponder_t));
+    rfid_transponder_t* transponder = rfid_transponder_alloc(&rfid);
     /* Fake manufacturer, unlikely to match an existing one to trigger failure. */
-    info->manufacturer = 1;
-    if(rfid_write(&rfid, &transponder) >= 0 ||
+    rfid_transponder_id_set(&rfid, transponder, "1.2.3.4.5.6.7.8");
+    if(rfid_write(&rfid, transponder) >= 0 ||
        rfid.error != EXDEV) {
       report_error(&rfid, "rfid_write (EXDEV was expected)");
+      rfid_transponder_free(&rfid, transponder);
       return -1;
     }
+    rfid_transponder_free(&rfid, transponder);
   }
   alarm(0);
   
@@ -199,10 +188,10 @@ bytes_per_block = %d\n\
     return -1;
   }
 
-  alarm(5);
+  alarm(timeout);
   {
     int count = 3;
-    if(rfid_read(&rfid, callback, &count) < 0) {
+    if(rfid_event_loop(&rfid, RFID_EVENT_NEW|RFID_EVENT_AGAIN, transponder_callback, 0, &count) < 0) {
       report_error(&rfid, "rfid_read");
       return -1;
     }
